<?php if (!defined('phpTodo')) exit();
/* phpTodo - Web based todo lists                                             *}
{* Copyright (C) 2005-07  Jason Frisvold <friz@godshell.com>                  *}
{*                                                                            *}
{* This program is free software; you can redistribute it and/or modify       *}
{* it under the terms of the GNU General Public License as published by       *}
{* the Free Software Foundation; either version 2 of the License, or          *}
{* (at your option) any later version.                                        *}
{*                                                                            *}
{* This program is distributed in the hope that it will be useful,            *}
{* but WITHOUT ANY WARRANTY; without even the implied warranty of             *}
{* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *}
{* GNU General Public License for more details.                               *}
{*                                                                            *}
{* You should have received a copy of the GNU General Public License          *}
{* along with this program; if not, write to the Free Software                *}
{* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */

/* $Id: classTodoItem.php,v 1.8 2007/04/06 17:54:16 xen0phage Exp $ */

// This is the phpTodo TodoItem class.
class TodoItem {
   // Class variables
   var $task_id;        // Integer
   var $user_id;        // Integer
   var $subject;        // Char(75)
   var $start_date;     // DateTime
   var $due_date;       // DateTime
   var $create_date;    // DateTime
   var $complete_date;  // DateTime
   var $priority_id;    // TinyInt
   var $status_id;      // TinyInt
   var $category_id;    // Integer
   var $last_modified;  // Timestamp
   var $description;    // Text
   var $nextaction;     // Text
   var $user_obj;       // Object

   var $dirty_flag;     // Flag to indicate that serialization is required
   var $dna_flag;       // Description / NextAction load flag
   var $delete_flag;    // Flag to mark an entry to be deleted

   // Class Constructor
   function TodoItem(&$user_obj, $task_id = -1) {
      // Make sure we have a valid user object
      if (! is_object($user_obj)) {
         $this->errstr = 'Invalid User Object';
         return false;
      }

      // Assign the user object (this *should* be a reference)
      $this->user_obj = $user_obj;

      // Everyone gets to start with a clean slate!
      $this->dirty_flag = false;
      $this->delete_flag = false;
      $this->dna_flag = false;

      // If we get a task_id, load the relevant data from the database
      if (is_numeric($task_id) && ($task_id != -1)) {

         // Get the task entry, ensuring at the same time that we have the
          // correct user
          $query = sprintf('SELECT subject, ' .
                           'DATE_FORMAT(start_date, "%s"), ' .
                           'DATE_FORMAT(due_date, "%s"), ' .
                           'DATE_FORMAT(create_date, "%s"), ' .
                           'DATE_FORMAT(complete_date, "%s"), ' .
                           'DATE_FORMAT(last_modified, "%s"), ' .
                           'priority_id, status_id, category_id FROM ' .
                           'listentries WHERE id = %d AND user_id = %d',
                           '%m-%d-%Y', '%m-%d-%Y', '%m-%d-%Y', '%m-%d-%Y',
                           '%m-%d-%Y', $task_id, $this->user_obj->user_id());
          $result = mysql_query($query);

          list($subject, $start_date, $due_date, $create_date, $complete_date,
               $last_modified, $priority_id, $status_id, $category_id) =
             mysql_fetch_array($result, MYSQL_NUM);
          mysql_free_result($result);

         $this->task_id       = $task_id;
         $this->user_id       = $this->user_obj->user_id();
         $this->start_date    = $start_date;
         $this->due_date      = $due_date;
         $this->create_date   = $create_date;
         $this->complete_date = $complete_date;
         $this->priority_id   = $priority_id;
         $this->status_id     = $status_id;
         $this->category_id   = $category_id;
         $this->subject       = $subject;
         $this->last_modified = $last_modified;

      } else {

         // If we do not get a task_id, then this is a new task
         $this->task_id       = -1;
         $this->user_id       = $this->user_obj->user_id();

         // Set these to 0 here - Load the relevant values if they are needed
         // later
         $this->priority_id   = 0;
         $this->status_id     = 0;
         $this->category_id   = 0;

         // This is a new task entry, mark as dirty
         $this->dirty_flag = true;
      }
   }

   // Return the task ID
   function task_id() {
      return $this->task_id;
   }

   // Set / Return the user_id assigned to this task
   function user_id($user_id = -1) {
      if (is_numeric($user_id) && ($user_id > 0)) {
         $this->user_id = $user_id;
         $this->dirty_flag = true; // Mark as dirty
      }

      return $this->user_id;
   }

   // Set / Return the subject
   function subject($subject = -1) {
      if ((func_num_args() > 0) && ($subject != -1)) {
         if (strlen($subject) <= 75) {
            $this->subject = urlencode($subject);
         } else {
            $this->subject = urlencode(substr($subject, 0, 75));
         }
         $this->dirty_flag = true; // Mark as dirty
      }

      // Determine if magic quotes is on and act appropriately
      if (get_magic_quotes_gpc()) {
         return urldecode(stripslashes($this->subject));
      } else {
         return urldecode($this->subject);
      }
   }

   function start_date($start_date = -1) {
      // We either want a date that is blank (setting to 0)
      // Or we want a well-formed date
      if (($start_date == '') || (($start_date != -1) &&
          preg_match('/^(?:[0-9]{1,2}-){2}[0-9]{4}\z/', $start_date))) {
         if ($start_date == '') {
            $this->start_date = '00-00-0000';
         } else {
            $this->start_date = $start_date;
         }
         $this->dirty_flag = true; // Mark as dirty
      }

      if ($this->start_date == '00-00-0000') {
         return;
      } else {
         return $this->start_date;
      }
   }

   // Set / Return the due_date
   function due_date($due_date = -1) {
      // We either want a date that is blank (setting to 0)
      // Or we want a well-formed date
      if (($due_date == '') || (($due_date != -1) &&
          preg_match('/^(?:[0-9]{1,2}-){2}[0-9]{4}\z/', $due_date))) {
         if ($due_date == '') {
            $this->due_date = '00-00-0000';
         } else {
            $this->due_date = $due_date;
         }

         $this->dirty_flag = true; // Mark as dirty
      }

      if ($this->due_date == '00-00-0000') {
         return;
      } else {
         return $this->due_date;
      }
   }

   // Set / Return the complete_date
   function complete_date($complete_date = -1) {
      // We either want a date that is blank (setting to 0)
      // Or we want a well-formed date
      if (($complete_date == '') || (($complete_date != -1) &&
          preg_match('/^(?:[0-9]{1,2}-){2}[0-9]{4}\z/', $complete_date))) {
         if ($complete_date == '') {
            $this->complete_date = '00-00-0000';
         } else {
            $this->complete_date = $complete_date;
         }
         $this->dirty_flag = true; // Mark as dirty
      }

      return $this->complete_date;
   }

   // Return the create date
   function create_date() {
      return $this->create_date;
   }

   // Return the last_modified date
   function last_modified() {
      return $this->last_modified;
   }

   // Set / Return the priority_id
   function priority_id($priority_id = -1) {
      if (is_numeric($priority_id) && ($priority_id > 0)) {
         $this->priority_id = $priority_id;
         $this->dirty_flag = true; // Mark as dirty
      }

      return $this->priority_id;
   }

   function status_id($status_id = -1) {
      if (is_numeric($status_id) && ($status_id > 0)) {
         $this->status_id = $status_id;
         $this->dirty_flag = true; // Mark as dirty
      }

      return $this->status_id;
   }

   function category_id($category_id = -1) {
      if (is_numeric($category_id) && ($category_id > 0)) {
         $this->category_id = $category_id;
         $this->dirty_flag = true; // Mark as dirty
      }

      return $this->category_id;
   }

   // Set / Return the description
   function description($description = -1) {
      if (! $this->dna_flag) {
         $this->_load_dna();
      }

      if ((func_num_args() > 0) && ($description != -1)) {
         $this->description = urlencode($description);
         $this->dirty_flag = true; // Mark as dirty
      }

      // Determine if magic quotes is on and act appropriately
      if (get_magic_quotes_gpc()) {
         return urldecode(stripslashes($this->description));
      } else {
         return urldecode($this->description);
      }
   }

   // Set / Return the next action
   function nextaction($nextaction = -1) {
      if (! $this->dna_flag) {
         $this->_load_dna();
      }

      if ((func_num_args() > 0) && ($nextaction != -1)) {
         $this->nextaction = urlencode($nextaction);
         $this->dirty_flag = true; // Mark as dirty
      }

      // Determine if magic quotes is on and act appropriately
      if (get_magic_quotes_gpc()) {
         return urldecode(stripslashes($this->nextaction));
      } else {
         return urldecode($this->nextaction);
      }
   }

   // Load the description and next action from the database
   function _load_dna() {
      if ($this->task_id != -1) {
         $query = sprintf('SELECT description, nextaction FROM description WHERE list_id = %d', $this->task_id);
         $result = mysql_query($query);
         list ($description, $nextaction) = mysql_fetch_array($result);

         $this->description = $description;
         $this->nextaction = $nextaction;
      }

      // Set the flag indicating that the data was loaded
      $this->dna_flag = true;

      return true;
   }

   // Set the delete flag
   function delete() {
      $this->delete_flag = true;

      return true;
   }

   // Return the error string (if there is one)
   function errstr() {
      return $this->errstr;
   }

   // Save / Delete data based on flags and content
   function _serialize() {
      // Pad the dates out so they match what mysql needs
      // Example : 11-23-2005 00:00:00
      $date_match = '/^([0-9]{1,2})-([0-9]{1,2})-([0-9]{4})\z/';
      $time_match = '/^([0-9]{1,2}):([0-9]{1,2}):([0-9]{1,2})\z/';
      if ($this->due_date != '') {
         $date_parts = explode(' ', $this->due_date);
         preg_match($date_match, $date_parts[0], $date_array);
         $due_date = $date_array[3];
         $due_date .= str_pad($date_array[1], 2, '0', STR_PAD_LEFT);
         $due_date .= str_pad($date_array[2], 2, '0', STR_PAD_LEFT);
         if (isset($date_parts[1]) && ($date_parts[1] != '00:00:00')) {
            preg_match($time_match, $date_parts[1], $time_array);
            $due_date .= str_pad($time_array[1], 2, '0', STR_PAD_LEFT);
            $due_date .= str_pad($time_array[2], 2, '0', STR_PAD_LEFT);
            $due_date .= str_pad($time_array[3], 2, '0', STR_PAD_LEFT);
         } else {
            $due_date .= "000000";
         }
      } else {
         $due_date = 'NULL';
      }

      if ($this->start_date != '') {
         $date_parts = explode(' ', $this->start_date);
         preg_match($date_match, $date_parts[0], $date_array);
         $start_date = $date_array[3];
         $start_date .= str_pad($date_array[1], 2, '0', STR_PAD_LEFT);
         $start_date .= str_pad($date_array[2], 2, '0', STR_PAD_LEFT);
         if (isset($date_parts[1]) && ($date_parts[1] != '00:00:00')) {
            preg_match($time_match, $date_parts[1], $time_array);
            $start_date .= str_pad($time_array[1], 2, '0', STR_PAD_LEFT);
            $start_date .= str_pad($time_array[2], 2, '0', STR_PAD_LEFT);
            $start_date .= str_pad($time_array[3], 2, '0', STR_PAD_LEFT);
         } else {
            $start_date .= "000000";
         }
      } else {
         $start_date = 'NULL';
      }

      // Compare the dates - Set the due date to null if it's before
      // the start date (assuming the start date exists)
      if (isset($start_date) && ($start_date != 'NULL')) {
         // Check Year
         if (substr($start_date, 0, 4) <= substr($due_date, 0, 4)) {
            // Check Month
            if (substr($start_date, 4, 2) <= substr($due_date, 4, 2)) {
               // Check Date
               if (substr($start_date, 6, 2) <= substr($due_date, 6, 2)) {
                  // Check Hour
                  if (substr($start_date, 8, 2) <= substr($due_date, 8, 2)) {
                     // Check Minute
                     if (substr($start_date, 10, 2) <= substr($due_date, 10, 2)) {
                        // Check Seconds
                        if (substr($start_date, 12, 2) > substr($due_date, 12, 2)) {
                           $due_date = 'NULL';
                        }
                     } else {
                        $due_date = 'NULL';
                     }
                  } else {
                     $due_date = 'NULL';
                  }
               } else {
                  $due_date = 'NULL';
               }
            } else {
               $due_date = 'NULL';
            }
         } else {
            $due_date = 'NULL';
         }
      }

      // If the delete flag is set, we need to delete the task
      if ($this->delete_flag) {
         // First make sure there's a task id
         if ($this->task_id != -1) {
            // Delete the entry, being careful to ensure it's from the right user
            $query = sprintf('DELETE FROM listentries, description USING ' .
                             'listentries LEFT JOIN description ON ' .
                             'description.list_id = listentries.id WHERE ' .
                             'listentries.id = %d AND listentries.user_id = %d',
                             $this->task_id, $this->user_obj->user_id());
            mysql_query($query) or die('Query Error: ' . mysql_error());
         }
      // If we don't have a task_id, this is a new task
      } else if ($this->task_id == -1) {
         // Create the query to enter the data into the database and enter it
         $query = sprintf('INSERT INTO listentries (user_id, subject, ' .
                          'start_date, due_date, priority_id, status_id, ' .
                          'category_id, create_date, last_modified) VALUES ' .
                          '(%d, "%s", %s, %s, %d, %d, %d, NOW(), NULL)',
                          $this->user_obj->user_id(), $this->subject, $start_date,
                          $due_date, $this->priority_id, $this->status_id,
                          $this->category_id);

         mysql_query($query) or die('Query Error: ' . mysql_error());

         // If the dnsflag is set, then there is a description/nextaction
         if ($this->dna_flag) {
            // Insert the description into the database
            $query = sprintf('INSERT INTO description (list_id, description, ' .
                             'nextaction) VALUES (%d, "%s", "%s")',
                             mysql_insert_id(), $this->description,
                             $this->nextaction);
            mysql_query($query);
         }
      // Is the task_id is set, then this is a pre-existing task
      } else {
         // Update the task in the database
         $query = sprintf('UPDATE listentries SET subject = "%s", due_date = ' .
                          '%s, start_date = "%s", priority_id = %d, ' .
                          'status_id = %d, category_id = %d, last_modified = ' .
                          'NULL WHERE id = %d AND user_id = %d', $this->subject,
                          $due_date, $start_date, $this->priority_id,
                          $this->status_id, $this->category_id, $this->task_id,
                          $this->user_obj->user_id());
         mysql_query($query);

         $info = mysql_info();
         ereg("Rows matched: ([0-9]*)", $info, $rows_matched);

         if ($rows_matched > 0) {
            // Find the description in the database
            $query = sprintf('SELECT id FROM description WHERE list_id = %d',
                             $this->task_id);

            $result = mysql_query($query) or die( 'Error: ' . mysql_error() );

            // If there is no current description, create an entry
            if (mysql_num_rows(mysql_query($query)) == 0) {
               $query = sprintf('INSERT INTO description (id, list_id, ' .
                                'description, nextaction) VALUES (NULL, %d, ' .
                                '"%s", "%s")', $this->task_id,
                                $this->description, $this->nextaction);

               mysql_query($query) or die( 'Error: ' . mysql_error() );
            // If there is a single description, update it
            } else if (mysql_num_rows(mysql_query($query)) == 1) {
               list($desc_id) = mysql_fetch_row($result);

               $query = sprintf('REPLACE INTO description (id, list_id, ' .
                                'description, nextaction) VALUES (%d, %d, ' .
                                '"%s", "%s")', $desc_id, $this->task_id,
                                $this->description, $this->nextaction);

               mysql_query($query) or die( 'Error: ' . mysql_error() );
            // If there are multiples, delete them all and insert this one
            // Note : This should *not* happen
            } else {
               $query = sprintf('DELETE FROM description WHERE list_id = %d',
                                $this->task_id);

               mysql_query($query) or die( 'Error: ' . mysql_error() );

               $query = sprintf('INSERT INTO description (id, list_id, ' .
                                'description, nextaction) VALUES (NULL, %d, ' .
                                '"%s", "%s")', $this->task_id,
                                $this->description, $this->nextaction);

               mysql_query($query) or die( 'Error: ' . mysql_error() );
            }

            if (isset($result)) {
               mysql_free_result($result);
            }
         }
      }
   }

   // Shutdown function
   function finish() {
      if ($this->dirty_flag) {
         $this->_serialize();
      }

      return true;
   }
}


?>
