<?php

   // ==================================================================
   //  Author: Ted Kappes (pesoto74@soltec.net)
   //	 Web: 	http://tkap.org/paginator/
   //	 Name: 	Paginator
   //  Desc: 	Class to help make pagination more easy.
   //
   // 7/21/2003
   //
   //  Please send me a mail telling me what you think of Paginator
   //  and what your using it for. [ pesoto74@soltec.net]
   //
   // ChangeLog :
   //    JHF - 12/27/06
   //       - Cleaned up general code layout
   //       - Added additional comments
   //       - Modified constructor to make total items optional
   //       - Added method to set total items
   //       - Added input validation
   //
   // ==================================================================

class Paginator {
   // All variables are pivate.
   var $previous;	
   var $next;
   var $page;
   var $total_pages;
   var $link_arr;
   var $range1;
   var $range2;
   var $last;
   var $first_of;
   var $second_of;
   var $limit;
   var $prev_next;
   var $base_page_num;
   var $extra_page_num;
   var $total_items;

   // Constructor for Paginator.  Takes the current page and (optionally) the
   // number of items in the source data and sets the current page
   // ($this->page) and the total items in the source ($this->total_items)
   function Paginator($page = 1, $num_rows = 0) {
      if (is_numeric($page) && ($page > 0)) {
         $this->page = $page;
      } else {
         $this->page = 1;
      }

      if (is_numeric($num_rows) && ($num_rows > 0)) {
         $this->total_items = $num_rows;
      }
   }

   // Sets the total number of items to be displayed
   function setTotalItems($num_rows) {
      if (is_numeric($num_rows) && ($num_rows > 0)) {
         $this->total_items = $num_rows;
      }
   }

   // Takes $limit and sets $this->limit. Calls private mehods
   // setBasePage() and setExtraPage() which use $this->limit.
   // Limit is the number of items to display per page
   function set_Limit($limit = 5) {
      if (is_numeric($limit) && ($limit > 0)) {
         $this->limit = $limit;
      } else {
         $this->limit = 5;
      }

      $this->setBasePage();
      $this->setExtraPage();
   }

   // This method creates a number that setExtraPage() uses to if there are
   // and extra pages after limit has divided the total number of pages.
   // base_page_num is the total number of complete pages (minus overflow)
   function setBasePage() {
      $div = $this->total_items / $this->limit;	
      $this->base_page_num = floor($div);
   }

   // extra_page_num is the total number of leftover items after all complete
   // pages are displayed (used as the "final" limit)
   function setExtraPage() {
      $this->extra_page_num =
         $this->total_items - ($this->base_page_num * $this->limit);
   }

   // Used in making numbered links.  Sets the number of links behind and 
   // ahead of the current page.  For example if there were a possiblity of
   // 20 numbered links and this was set to 5 and the current link was 10
   // the result would be this 5 6 7 8 9 10 11 12 13 14 15
   function set_Links($prev_next = 5) {
      if (is_numeric($prev_next) && ($prev_next > 0)) {
         $this->prev_next = $prev_next;
      } else {
         $this->prev_next = 1;
      }
   }

   // Method to get the total items.
   function getTotalItems() {
      return $this->total_items;
   }

   // Method to get the base number to use in queries and such
   function getRange1() {
      $this->range1 = ($this->limit * $this->page) - $this->limit;	
      return $this->range1;
   }

   // Method to get the offset for use in queries
   function getRange2() {
      if ($this->page == $this->base_page_num + 1) {
         $this->range2 = $this->extra_page_num;
      } else {
         $this->range2 = $this->limit;
      }
      return $this->range2;
   }

   // Method to get the first of number as in 5 of 8
   function getFirstOf() {
      $this->first_of = $this->range1 + 1;
      return $this->first_of;
   }

   // Method to get the second number in a series as in 5 of 8
   function getSecondOf() {
      if($this->page == $this->base_page_num + 1) {
         $this->second_of = $this->range1 + $this->extra_page_num;
      } else {
         $this->second_of = $this->range1 + $this->limit;
      }
      return $this->second_of;
   }

   // Method to get the total number of pages.
   function getTotalPages() {
      if ($this->extra_page_num) {
         $this->total_pages = $this->base_page_num + 1;
      } else {
         $this->total_pages = $this->base_page_num;
      }
      return $this->total_pages;
   }

   // Method to get the first link number.
   function getFirst() {
      return 1;
   }

   // Method to get the last link number.
   function getLast() {
      if ($this->page == $this->total_pages) {
         $this->last = 0;
      } else {
         $this->last = $this->total_pages;
      }
      return $this->last;  
   }

   // Method to get the number of the link previous to the current link
   function getPrevious() {
      if($this->page > 1) {
         $this->previous = $this->page - 1;
      } else {
         $this->previous = 1;
      }
      return $this->previous;
   }

   // Method to get the number of the current link
   function getCurrent() {
      return $this->page;
   }

   // Method to get the current page name, mostly used in links to the next page
   function getPageName() {
      return $_SERVER['PHP_SELF'];
   }

   // Method to get the number of the link after the current link
   function getNext() {   
      $this->getTotalPages();
      if ($this->total_pages != $this->page) {
         $this->next = $this->page + 1;
      }
      return $this->next;
   }

   // Method that returns an array of the numbered links that should be
   // displayed
   function getLinkArr() {
      // Gets the top range   
      $top = $this->getTotalPages() - $this->getCurrent();
      if($top <= $this->prev_next) {
         $top = $top;
         $top_range = $this->getCurrent() + $top;
      } else {
         $top = $this->prev_next;
         $top_range = $this->getCurrent() + $top;
      }
      
      // Gets the bottom range
      $bottom = $this->getCurrent() - 1;
      if($bottom <= $this->prev_next) {
         $bottom = $bottom;
         $bottom_range = $this->getCurrent() - $bottom;
      } else {
         $bottom = $this->prev_next;
         $bottom_range = $this->getCurrent() - $bottom;
      } 
      
      $j=0;
      foreach(range($bottom_range, $top_range) as $i) {
         $this->link_arr[$j] = $i;
         $j++;
      }
      return $this->link_arr;
   }

} // ends Paginator class

?>	
