<?php if (!defined('phpTodo')) exit();
/* phpTodo - Web based todo lists                                             *}
{* Copyright (C) 2005-07  Jason Frisvold <friz@godshell.com>                  *}
{*                                                                            *}
{* This program is free software; you can redistribute it and/or modify       *}
{* it under the terms of the GNU General Public License as published by       *}
{* the Free Software Foundation; either version 2 of the License, or          *}
{* (at your option) any later version.                                        *}
{*                                                                            *}
{* This program is distributed in the hope that it will be useful,            *}
{* but WITHOUT ANY WARRANTY; without even the implied warranty of             *}
{* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *}
{* GNU General Public License for more details.                               *}
{*                                                                            *}
{* You should have received a copy of the GNU General Public License          *}
{* along with this program; if not, write to the Free Software                *}
{* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */

/* $Id: phpTodoUser.php,v 1.18 2007/04/06 17:54:16 xen0phage Exp $ */

// This is the phpTodo User class.
class phpTodoUser {
   // Class variables
   var $user_id;
   var $username;
   var $fullname;
   var $email;
   var $adminflag;
   var $logged_in;

   var $priority_sort;
   var $status_sort;
   var $duedate_sort;
   var $subject_sort;
   var $category_filter;
   var $show_complete;
   var $categories;
   var $sort_stabilizer;
   var $sort_order;
   var $_category_change;
   var $_cat_load;

   var $dirty_flag;
   var $pwd_dirty_flag;
   var $category_dirty_flag;
   var $prefs_dirty_flag;

   var $deleteme;
   
   // Class Constructor
   function phpTodoUser($uid = -1, $username = '', $password = '',
                        $login = false) {

      // Everyone starts with a clean slate
      $this->dirty_flag = false;
      $this->pwd_dirty_flag = false;
      $this->prefs_dirty_flag = false;
      $this->category_dirty_flag = false;

      // Don't delete unless explicitly told to
      $this->deleteme = false;

      $this->_cat_load = false;

      // All users are considered offline until otherwise determined
      $this->logged_in = false;

      // Check to see if this is a login attempt
      if ($login) {
         // Input checking for username and password
         $phpTodo_nameRegex = '/^[a-zA-Z0-9_\-]{1,15}\z/';
         $phpTodo_pwdRegex = '/^[a-zA-Z0-9@#$%\^&\*\/]{4,15}\z/';
         $clean_username = '';
         $clean_password = '';

         if (preg_match($phpTodo_nameRegex, $username)) {
            $clean_username = $username;
         }

         if (preg_match($phpTodo_pwdRegex, $password)) {
            $clean_password = $password;
         }

         // Check the database for the user and get their password
         $query = sprintf('SELECT id FROM users WHERE username = "%s" ' .
                          'AND password = SHA1("%s")', $clean_username,
                          mysql_real_escape_string($clean_password));

         $result = mysql_query($query);

         // If a single row was returned then we have a match
         if (mysql_num_rows($result) == 1) {
            // Get the user_id from the sql query result
            list($this->user_id) = mysql_fetch_array($result);

            mysql_free_result($result);

            // Mark the user as logged in
            $this->logged_in = true;
         }
      }

      // If we have a UID, or we are logged in
      if ((is_numeric($uid) && ($uid != -1)) || $this->logged_in) {
         // Assign the UID if needed
         if (! $this->logged_in) {
            $this->user_id = $uid;
         }

         // Create the necessary sql statement
         $query = sprintf('SELECT u.username, u.full_name, u.email, ' .
                          'u.admin, up.priority_sort, up.status_sort, ' .
                          'up.subject_sort, up.duedate_sort, up.category_filter, ' .
                          'up.show_complete, up.sort_stabilizer, up.sort_order ' .
                          'FROM users AS u LEFT JOIN user_prefs AS up ON ' .
                          'up.user_id = u.id WHERE u.id = %d', $this->user_id);

         $result = mysql_query($query) or die( 'Error: ' . mysql_error() );

         $line = mysql_fetch_array($result, MYSQL_NUM);

         $this->username = $line[0];
         $this->fullname = $line[1];
         $this->email = $line[2];
         $this->adminflag = $line[3];
         $this->priority_sort = $line[4];
         $this->status_sort = $line[5];
         $this->subject_sort = $line[6];
         $this->duedate_sort = $line[7];
         $this->category_filter = $line[8];
         $this->show_complete = $line[9];
         $this->sort_stabilizer = $line[10];
         $this->sort_order = $line[11];

      // New user, use defaults
      } else {
         $this->user_id = -1;
         $this->username($username);
         $this->fullname = '';
         $this->adminflag = 0;

         $this->priority_sort = 0;
         $this->status_sort = 0;
         $this->duedate_sort = 0;
         $this->subject_sort = 0;
         $this->category_filter = 0;
         $this->show_complete = 0;
         $this->sort_stabilizer = 'none';
         $this->sort_order = 'asc';

         // Mark the object as dirty
         $this->dirty_flag = true;
      }
   }

   function user_id() {
      if ($this->user_id == -1) {
         $this->_serialize();
      }

      return $this->user_id;
   }

   function logged_in() {
      return $this->logged_in;
   }

   function password($password = '') {
      $phpTodo_pwdRegex = '/^[a-zA-Z0-9@#$%\^&\*\/]{4,15}\z/';
      if (! empty($password)) {
         if (preg_match($phpTodo_pwdRegex, $password)) {
            $this->password = $password;

            // Mark as dirty
            $this->pwd_dirty_flag = true;
         } else {
            return false;
         }
      }

      return $this->password;
   }

   function check_pwd($password = '') {
      $phpTodo_pwdRegex = '/^[a-zA-Z0-9@#$%\^&\*\/]{4,15}\z/';
      if (! empty($password) && preg_match($phpTodo_pwdRegex, $password)) {
         // Check the password against whats stored in the database
         $query = sprintf('SELECT count(*) FROM users WHERE id = %d ' .
                          'AND password = SHA1("%s")', $this->user_id,
                          mysql_real_escape_string($password));

         list($count) = mysql_fetch_array(mysql_query($query));
         // If a single row was returned then we have a match
         if ($count == 1) {
            return true;
         }
      }

      // If we get here, return a failure
      return false;
   }

   function username($username = '') {
      $phpTodo_nameRegex = '/^[a-zA-Z0-9_\-]{1,15}\z/';
      if (! empty($username) && (! strcmp($username, $this->username) == 0)) {
         if (preg_match($phpTodo_nameRegex, $username) && ($username != 'guest')) {
            $this->username = $username;

            // Mark as dirty
            $this->dirty_flag = true;
         } else {
            return false;
         }
      }

      return $this->username;
   }

   function fullname($fullname = '') {
      $phpTodo_fullnameRegex = '/^[a-zA-Z0-9_\-\ \']{1,40}\z/';
      if (! empty($fullname) && (! strcmp($fullname, $this->fullname) == 0)) {
         if (preg_match($phpTodo_fullnameRegex, $fullname)) {
            $this->fullname = $fullname;

            // Mark as dirty
            $this->dirty_flag = true;
         } else {
            return false;
         }
      }

      return $this->fullname;
   }

   function email($email = '') {
      $phpTodo_emailRegex = '/^[a-zA-Z0-9_\-\.!#\$%&\*\+\/=\?\^\{\|\}~]{0,64}@[a-zA-Z0-9\-\.]{0,255}\z/';
      if (! empty($email) && (! strcmp($email, $this->email) == 0)) {
         if (preg_match($phpTodo_emailRegex, $email)) {
            $this->email = $email;

            // Mark as dirty
            $this->dirty_flag = true;
         } else {
            return false;
         }
      }

      return $this->email;
   }

   function adminflag($adminflag = '') {
      if (! empty($adminflag) && ($this->adminflag != $adminflag)) {
         if (is_bool($adminflag)) {
            $this->adminflag = $adminflag;

            // Mark as dirty
            $this->dirty_flag = true;
         } else {
            return false;
         }
      }

      return $this->adminflag;
   }

   function priority_sort($priority_sort = '') {
      if (is_numeric($priority_sort) && ($this->priority_sort != $priority_sort)) {
         if ($priority_sort >= 0) {
            $this->priority_sort = $priority_sort;

            // Mark as dirty
            $this->prefs_dirty_flag = true;
         } else {
            return false;
         }
      }

      return $this->priority_sort;
   }

   function status_sort($status_sort = '') {
      if (is_numeric($status_sort) && ($this->status_sort != $status_sort)) {
         if ($status_sort >= 0) {
            $this->status_sort = $status_sort;

            // Mark as dirty
            $this->prefs_dirty_flag = true;
         } else {
            return false;
         }
      }

      return $this->status_sort;
   }

   function subject_sort($subject_sort = '') {
      if (is_numeric($subject_sort) && ($this->subject_sort != $subject_sort)) {
         if ($subject_sort >= 0) {
            $this->subject_sort = $subject_sort;

            // Mark as dirty
            $this->prefs_dirty_flag = true;
         } else {
            return false;
         }
      }

      return $this->subject_sort;
   }

   function duedate_sort($duedate_sort = '') {
      if (is_numeric($duedate_sort) && ($this->duedate_sort != $duedate_sort)) {
         if ($duedate_sort >= 0) {
            $this->duedate_sort = $duedate_sort;

            // Mark as dirty
            $this->prefs_dirty_flag = true;
         } else {
            return false;
         }
      }

      return $this->duedate_sort;
   }

   function category_filter($category_filter = '') {
      if (is_numeric($category_filter) &&
          ($this->category_filter != $category_filter)) {
         if ($category_filter >= -1) {
            $this->category_filter = $category_filter;

            // Mark as dirty
            $this->prefs_dirty_flag = true;
         } else {
            return false;
         }
      }

      return $this->category_filter;
   }

   function show_complete($show_complete = '') {
      if (is_bool($show_complete) && ($this->show_complete != $show_complete)) {
         $this->show_complete = $show_complete;

         // Mark as dirty
         $this->prefs_dirty_flag = true;
      }

      return $this->show_complete;
   }

   function sort_stabilizer($sort_stabilizer = '') {
      if (! empty($sort_stabilizer) &&
          ($this->sort_stabilizer != $sort_stabilizer)) {
         if (preg_match('/^(none|subject|lastmodified|startdate|duedate|' .
                        'createdate|priority|status|taskid)\z/',
                        $sort_stabilizer)) {
            $this->sort_stabilizer = $sort_stabilizer;

            // Mark as dirty
            $this->prefs_dirty_flag = true;
         } else {
            return false;
         }
      }

      return $this->sort_stabilizer;
   }

   function sort_order($sort_order = '') {
      if (! empty($sort_order) && ($this->sort_order != $sort_order)) {
         if (preg_match('/^(asc|dsc)\z/', $sort_order)) {
            $this->sort_order = $sort_order;

            // Mark as dirty
            $this->prefs_dirty_flag = true;
         } else {
            return false;
         }
      }

      return $this->sort_order;
   }

   function categories($category = '', $delete = false) {
      $phpTodo_nameRegex = '/^[a-zA-Z0-9_\-]{1,15}\z/';
      if (! $this->_cat_load) {
         $this->_load_categories();
      }

      if ($delete && is_numeric($category)) {
         $this->_category_change[$category] = $category;
         unset($this->categories[$category]);

         // Mark as dirty
         $this->category_dirty_flag = true;
      } elseif (! empty($category) && preg_match($phpTodo_nameRegex, $category)) {
         $this->_category_change[0] = $category;

         $this->_serialize_categories();
         $this->_load_categories();
      }

      return $this->categories;
   }

   function delete() {
      $this->deleteme = true;
   }

   function _load_categories() {
      $query = sprintf('SELECT id, category FROM category WHERE user_id = %d ' .
                       'ORDER BY category', $this->user_id());

      $result = mysql_query($query) or die ( 'Error: Query: ' . mysql_error() );
      while ($line = mysql_fetch_array($result)) {
         $this->categories[$line[0]] = $line[1];
      }

      // Mark categories as loaded
      $this->_cat_load = true;
   }

   function _delete() {
            // If the deleteme flag is set, delete the user
      if ($this->deleteme) {
         // Delete the entries, being careful to ensure it's
         // from the right user
         $query = sprintf('DELETE FROM users WHERE id = %d',
                          $this->user_id);
         mysql_query($query) or die( 'Error:' . mysql_error() );

         $query = sprintf('DELETE FROM user_prefs WHERE user_id = %d',
                          $this->user_id);
         mysql_query($query) or die( 'Error:' . mysql_error() );

         $query = sprintf('DELETE FROM rss_feeds WHERE user_id = %d',
                          $this->user_id);
         mysql_query($query) or die( 'Error:' . mysql_error() );

         $query = sprintf('DELETE FROM category WHERE user_id = %d',
                          $this->user_id);
         mysql_query($query) or die( 'Error:' . mysql_error() );

         $query = sprintf('DELETE FROM listentries, description USING ' .
                          'listentries LEFT JOIN description ON ' .
                          'description.list_id = listentries.id WHERE ' .
                          'listentries.user_id = %d', $this->user_id);
         mysql_query($query) or die( 'Error:' . mysql_error() );
      }
   }

   function _serialize() {
      // username, fullname, password, admin flag
      // User ID of -1 indicates a new user
      if ($this->user_id == -1) {
         $query = sprintf('INSERT INTO users (username, full_name, email, ' .
                          'password, admin) VALUES ("%s", "%s", "%s", ' .
                          'SHA1("%s"), %d)', $this->username, $this->fullname,
                          mysql_real_escape_string($this->email),
                          $this->password, $this->adminflag);

         mysql_query($query) or die ('Error: Query: ' . mysql_error() );

         // Get the user_id
         $this->user_id = mysql_insert_id();

         // Create a prefs entry
         $query = sprintf('INSERT INTO user_prefs (user_id) VALUES (%d)',
                          $this->user_id);

         mysql_query($query) or die ('Error: Query: ' . mysql_error() );
      } else {
         $query = sprintf('UPDATE users SET username = "%s", full_name = ' .
                          '"%s", email = "%s", admin = %d WHERE id = %d',
                          $this->username, $this->fullname,
                          mysql_real_escape_string($this->email),
                          $this->adminflag, $this->user_id);

         mysql_query($query) or die ('Error: Query: ' . mysql_error() );

         if ($this->pwd_dirty_flag) {
            $query = sprintf('UPDATE users SET password = SHA1("%s") WHERE ' .
                             'id = %d', $this->password, $this->user_id);

            mysql_query($query) or die ('Error: Query: ' . mysql_error() );
         }
      }
   }

   function _serialize_categories() {
      if (isset($this->_category_change[0])) {
         $query = 
            sprintf('INSERT INTO category (id, category, user_id) ' .
                  'VALUES (NULL, "%s", %d)',
                  mysql_real_escape_string($this->_category_change[0]),
                  $this->user_id());
         mysql_query($query) or die ('Error: Query: ' . mysql_error() );
      } else {
         $query = sprintf('DELETE FROM category WHERE id IN (%s) AND ' .
                          'user_id = %d',
                          implode(',', array_keys($this->_category_change)),
                          $this->user_id());
         mysql_query($query) or die ('Error: Query: ' . mysql_error() );

         // Set the category_id for existing list entries to 1 if the
         // original category_id was just deleted
         $query = sprintf('UPDATE listentries SET category_id = 1 WHERE ' .
                          'user_id = %d AND category_id IN (%s)',
                          $this->user_id(),
                          implode(',', array_keys($this->_category_change)));
         mysql_query($query) or die( 'Error: ' . mysql_error() );
      }
   }

   function _serialize_prefs() {
      // Update the user_prefs database
      $query = sprintf('UPDATE user_prefs SET priority_sort = %d, ' .
                       'status_sort = %d, subject_sort = %d, duedate_sort ' .
                       '= %d, category_filter = %d, show_complete = %d, ' .
                       'sort_stabilizer = "%s", sort_order = "%s" WHERE ' .
                       'user_id = %d', $this->priority_sort,
                       $this->status_sort, $this->subject_sort,
                       $this->duedate_sort, $this->category_filter,
                       $this->show_complete, $this->sort_stabilizer,
                       $this->sort_order, $this->user_id);

      mysql_query($query) or die( 'Error: ' . mysql_error() );
   }

   // Shutdown function
   function finish() {
      if ($this->deleteme) {
         $this->_delete();
         return;
      }

      if ($this->dirty_flag || $this->pwd_dirty_flag) {
         $this->_serialize();
      }

      if ($this->prefs_dirty_flag) {
         $this->_serialize_prefs();
      }

      if ($this->category_dirty_flag) {
         $this->_serialize_categories();
      }

      return true;
   }
}

?>
